import logging
import os

from datetime import datetime

DEFAULT_PROJECT = "bg-review-file-utils"

DEFAULT_TIMESTAMP = str(datetime.today().strftime("%Y-%m-%d-%H%M%S"))

DEFAULT_OUTDIR_BASE = os.path.join(
    "/tmp/",
    os.getenv("USER"),
    DEFAULT_PROJECT,
)

DEFAULT_LOGGING_FORMAT = "%(levelname)s : %(asctime)s : %(pathname)s : %(lineno)d : %(message)s"

DEFAULT_LOGGING_LEVEL = logging.INFO

DEFAULT_VERBOSE = False


DEFAULT_LAB_NUMBER_IDX = 0
DEFAULT_OVERALL_CALL_IDX = 1
DEFAULT_GENE_IDX = 2
DEFAULT_ZYGOSITY_IDX = 3
DEFAULT_MUTATION_IDX = 4
DEFAULT_VARIANT_PHEN_IDX = 5
DEFAULT_DISEASE_IDX = 6
DEFAULT_INTERNAL_COMMENT_IDX = 7
DEFAULT_IS_ON_REPORT_IDX = 8
DEFAULT_SANGER_IDX = 9
DEFAULT_SCORE_IDX = 10
DEFAULT_VCF_IDX = 11
DEFAULT_INTERPRETATION_IDX = 12
DEFAULT_SOURCE_IDX = 13
DEFAULT_LINK_IDX = 14
DEFAULT_QUALITY_IDX = 15
DEFAULT_FORMAT_IDX = 16
DEFAULT_MRNA_IDX = 17
DEFAULT_CLINICAL_CATEGORY_IDX = 18
DEFAULT_CLINICAL_GENETICS_SUPPORTING_IDX = 19
DEFAULT_CLINICAL_GENETICS_DISPROVE_IDX = 20
DEFAULT_CLINICAL_GENETICS_INTERMEDIATE_IDX = 21
DEFAULT_POPULATION_GENETICS_SUPPORTING_IDX = 22
DEFAULT_POPULATION_GENETICS_DISPROVE_IDX = 23
DEFAULT_POPULATION_GENETICS_INTERMEDIATE_IDX = 24
DEFAULT_FUNCTIONAL_ASSAY_IDX = 25
DEFAULT_EXPERIMENTAL_SUPPORTING_IDX = 26
DEFAULT_EXPERIMENTAL_DISPROVE_IDX = 27
DEFAULT_EXPERIMENTAL_INTERMEDIATE_IDX = 28
DEFAULT_THEORETICAL_SUPPORTING_IDX = 29
DEFAULT_THEORETICAL_DISPROVE_IDX = 30
DEFAULT_THEORETICAL_INTERMEDIATE_IDX = 31
DEFAULT_UNNAMED_1_IDX = 32
DEFAULT_UNNAMED_2_IDX = 33
DEFAULT_UNNAMED_3_IDX = 34
DEFAULT_UNNAMED_4_IDX = 35
DEFAULT_GNOMAD_HET_AC_IDX = 36
DEFAULT_GNOMAD_HOMO_AC_IDX = 37
DEFAULT_GNOMAD_POPMAX_AF_IDX = 38
DEFAULT_GNOMAD_OVERALL_AF_IDX = 39
DEFAULT_CLINVAR_ID_IDX = 40
DEFAULT_CLINVAR_SIGNIFICANCE_IDX = 41
DEFAULT_CLINVAR_AMBRY_IDX = 42
DEFAULT_CLINVAR_INVITAE_IDX = 43
DEFAULT_CLINVAR_COUNSYL_IDX = 44
DEFAULT_CLINVAR_IGLCOA_IDX = 45
DEFAULT_CLINVAR_GENEDX_IDX = 46
DEFAULT_CLINVAR_NATERA_IDX = 47
DEFAULT_CADD_PHRED_SCORE_IDX = 48
DEFAULT_SPLICEAI_SCORE_IDX = 49
DEFAULT_EXON_NUMBER_IDX = 50
DEFAULT_FS_VALUE_IDX = 51
DEFAULT_HGMD_CLASS_IDX = 52
DEFAULT_HGMD_ID_IDX = 53
DEFAULT_HGMD_PHEN_IDX = 54
DEFAULT_HGMD_PMID_IDX = 55
DEFAULT_MUT_TYPE_IDX = 56
DEFAULT_BAM_LINK_IDX = 57
DEFAULT_IGV_LINK_IDX = 58
DEFAULT_PROXIMAL_IDX = 69


